@@ -1,9 +1,10 @@
 #' Interpolate strings with SQL escaping
 #'
+#' @description
 #' SQL databases often have custom quotation syntax for identifiers and strings
 #' which make writing SQL queries error prone and cumbersome to do. `glue_sql()` and
-#' `glue_data_sql()` are analogs to `glue()` and `glue_data()` which handle the
-#' SQL quoting.
+#' `glue_data_sql()` are analogs to [glue()] and [glue_data()] which handle the
+#' SQL quoting. `glue_sql_collapse()` can be used to collapse [DBI::SQL()] objects.
 #'
 #' They automatically quote character results, quote identifiers if the glue
 #' expression is surrounded by backticks '\verb{`}' and do not quote
@@ -11,10 +12,10 @@
 #' column (which should be quoted) pass the data to `glue_sql()` as a
 #' character.
 #'
-#' Returning the result with `DBI::SQL()` will suppress quoting if desired for
+#' Returning the result with [DBI::SQL()] will suppress quoting if desired for
 #' a given value.
 #'
-#' Note [parameterized queries](https://db.rstudio.com/best-practices/run-queries-safely#parameterized-queries)
+#' Note [parameterized queries](https://solutions.posit.co/connections/db/best-practices/run-queries-safely/#parameterized-queries)
 #' are generally the safest and most efficient way to pass user defined
 #' values in a query, however not every database driver supports them.
 #'
@@ -22,9 +23,15 @@
 #' collapsed with commas. This is useful for the [SQL IN Operator](https://www.w3schools.com/sql/sql_in.asp)
 #' for instance.
 #' @inheritParams glue
-#' @param .con \[`DBIConnection`]:A DBI connection object obtained from `DBI::dbConnect()`.
-#' @return A `DBI::SQL()` object with the given query.
-#' @examples
+#' @seealso [glue_sql_collapse()] to collapse [DBI::SQL()] objects.
+#' @param .con \[`DBIConnection`]: A DBI connection object obtained from
+#'   [DBI::dbConnect()].
+#' @param .na \[`character(1)`: `DBI::SQL("NULL")`]\cr Value to replace
+#'   `NA` values with. If `NULL` missing values are propagated, that is an `NA`
+#'   result will cause `NA` output. Otherwise the value is replaced by the
+#'   value of `.na`.
+#' @return A [DBI::SQL()] object with the given query.
+#' @examplesIf requireNamespace("DBI", quietly = TRUE) && requireNamespace("RSQLite", quietly = TRUE)
 #' con <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
 #' iris2 <- iris
 #' colnames(iris2) <- gsub("[.]", "_", tolower(colnames(iris)))
@@ -89,7 +96,7 @@
 #' glue_sql("SELECT * FROM {`tbl`} WHERE species IN ({vals*})",
 #'   vals = c("setosa", "versicolor"), .con = con)
 #'
-#' # If you need to reference a variables from multiple tables use `DBI::Id()`.
+#' # If you need to reference variables from multiple tables use `DBI::Id()`.
 #' # Here we create a new table of nicknames, join the two tables together and
 #' # select columns from both tables. Using `DBI::Id()` and the special
 #' # `glue_sql()` syntax ensures all the table and column identifiers are quoted
@@ -103,17 +110,17 @@
 #'   nickname = c("Beachhead Iris", "Harlequin Blueflag", "Virginia Iris"),
 #'   stringsAsFactors = FALSE
 #' )
-#' 
+#'
 #' DBI::dbWriteTable(con, nicknames_db, nicknames)
-#' 
+#'
 #' cols <- list(
-#'   DBI::Id(table = iris_db, column = "sepal_length"),
-#'   DBI::Id(table = iris_db, column = "sepal_width"),
-#'   DBI::Id(table = nicknames_db, column = "nickname")
+#'   DBI::Id(iris_db, "sepal_length"),
+#'   DBI::Id(iris_db, "sepal_width"),
+#'   DBI::Id(nicknames_db, "nickname")
 #' )
 #'
-#' iris_species <- DBI::Id(table = iris_db, column = "species")
-#' nicknames_species <- DBI::Id(table = nicknames_db, column = "species")
+#' iris_species <- DBI::Id(iris_db, "species")
+#' nicknames_species <- DBI::Id(nicknames_db, "species")
 #'
 #' query <- glue_sql("
 #'   SELECT {`cols`*}
@@ -128,14 +135,67 @@
 #'
 #' DBI::dbDisconnect(con)
 #' @export
-glue_sql <- function(..., .con, .envir = parent.frame(), .na = DBI::SQL("NULL")) {
-  DBI::SQL(glue(..., .envir = .envir, .na = .na, .transformer = sql_quote_transformer(.con, .na)))
+glue_sql <- function(...,
+                     .con,
+                     .sep = "",
+                     .envir = parent.frame(),
+                     .open = "{",
+                     .close = "}",
+                     .na = DBI::SQL("NULL"),
+                     .null = character(),
+                     .comment = "#",
+                     .literal = FALSE,
+                     .trim = TRUE
+                     ) {
+  DBI::SQL(glue(
+    ...,
+    .sep = .sep,
+    .envir = .envir,
+    .open = .open,
+    .close = .close,
+    .na = .na,
+    .null = .null,
+    .comment = .comment,
+    .literal = .literal,
+    .transformer = sql_quote_transformer(.con, .na),
+    .trim = .trim
+  ))
 }
 
 #' @rdname glue_sql
 #' @export
-glue_data_sql <- function(.x, ..., .con, .envir = parent.frame(), .na = DBI::SQL("NULL")) {
-  DBI::SQL(glue_data(.x, ..., .envir = .envir, .na = .na, .transformer = sql_quote_transformer(.con, .na)))
+glue_data_sql <- function(.x,
+                          ...,
+                          .con,
+                          .sep = "",
+                          .envir = parent.frame(),
+                          .open = "{",
+                          .close = "}",
+                          .na = DBI::SQL("NULL"),
+                          .null = character(),
+                          .comment = "#",
+                          .literal = FALSE,
+                          .trim = TRUE) {
+  DBI::SQL(glue_data(
+    .x,
+    ...,
+    .sep = .sep,
+    .envir = .envir,
+    .open = .open,
+    .close = .close,
+    .na = .na,
+    .null = .null,
+    .comment = .comment,
+    .literal = .literal,
+    .transformer = sql_quote_transformer(.con, .na),
+    .trim = .trim
+  ))
+}
+
+#' @rdname glue_collapse
+#' @export
+glue_sql_collapse <- function(x, sep = "", width = Inf, last = "") {
+  DBI::SQL(glue_collapse(x, sep = sep, width = width, last = last))
 }
 
 sql_quote_transformer <- function(connection, .na) {
@@ -144,15 +204,15 @@ sql_quote_transformer <- function(connection, .na) {
   }
 
   function(text, envir) {
-    should_collapse <- grepl("[*]$", text)
+    should_collapse <- grepl("[*][[:space:]]*$", text)
     if (should_collapse) {
-      text <- sub("[*]$", "", text)
+      text <- sub("[*][[:space:]]*$", "", text)
     }
     m <- gregexpr("^`|`$", text)
     is_quoted <- any(m[[1]] != -1)
     if (is_quoted) {
       regmatches(text, m) <- ""
-      res <- eval(parse(text = text, keep.source = FALSE), envir)
+      res <- identity_transformer(text, envir)
 
       if (length(res) == 1) {
         res <- DBI::dbQuoteIdentifier(conn = connection, res)
@@ -162,35 +222,31 @@ sql_quote_transformer <- function(connection, .na) {
         res[] <- lapply(res, DBI::dbQuoteIdentifier, conn = connection)
       }
     } else {
-      res <- eval(parse(text = text, keep.source = FALSE), envir)
+      res <- identity_transformer(text, envir)
+      if (length(res) == 0L) {
+        if (should_collapse) {
+          return("")
+        } else {
+          return(NULL)
+        }
+      }
       if (inherits(res, "SQL")) {
         if (should_collapse) {
           res <- glue_collapse(res, ", ")
         }
         return(res)
       }
-
-      # convert objects to characters
-      is_object <- is.object(res)
-      if (is_object) {
-        res <- as.character(res)
-      }
-
-      is_na <- is.na(res)
-      if (any(is_na)) {
-        res[is_na] <- rep(list(.na), sum(is_na))
-      }
-
-      is_char <- vapply(res, function(x) !is.na(x) && is.character(x), logical(1))
-      res[is_char] <- lapply(res[is_char], function(x) DBI::dbQuoteLiteral(conn = connection, x))
-      res[!is_char] <- lapply(res[!is_char], function(x) DBI::SQL(conn = connection, x))
     }
+
+    if (is.list(res)) {
+      res <- unlist(lapply(res, DBI::dbQuoteLiteral, conn = connection))
+    } else {
+      res <- DBI::dbQuoteLiteral(connection, res)
+    }
+
     if (should_collapse) {
       res <- glue_collapse(res, ", ")
     }
-    if (length(res) == 0L) {
-      res <- DBI::SQL("NULL")
-    }
     res
   }
 }
