@@ -48,6 +48,7 @@ SEXP trim_(SEXP x) {
     while (i < str_len) {
       if (xx[i] == '\n') {
         new_line = true;
+        indent = 0;
       } else if (new_line) {
         if (xx[i] == ' ' || xx[i] == '\t') {
           ++indent;
@@ -62,7 +63,8 @@ SEXP trim_(SEXP x) {
       ++i;
     }
 
-    if (new_line && indent < min_indent) {
+    /* if string ends with '\n', `indent = 0` only because we made it so */
+    if (xx[str_len - 1] != '\n' && new_line && indent < min_indent) {
       min_indent = indent;
     }
 
